@@ -1,12 +1,12 @@
 use std::ops::{Deref, Range};
-use std::sync::{LazyLock, RwLock};
+use std::sync::LazyLock;
 
 use anyhow::{Context, Result};
 use github_actions_models::action;
 use github_actions_models::workflow::job::StepBody;
 use regex::Regex;
 use tree_sitter::{
-    Language, Parser, Query, QueryCapture, QueryCursor, QueryMatches, StreamingIterator as _, Tree,
+    Language, Parser, QueryCapture, QueryCursor, QueryMatches, StreamingIterator as _, Tree,
 };
 
 use super::{Audit, AuditLoadError, audit_meta};
@@ -21,52 +21,18 @@ static GITHUB_ENV_WRITE_CMD: LazyLock<Regex> = LazyLock::new(|| {
 });
 
 pub(crate) struct GitHubEnv {
-    // Ideally we'd use a RefCell here, but the LSP mode requires that
-    // each Audit be Sync.
-    // TODO: Evaluate whether this is worth it. Maybe just loading a new parser
-    // each time would be just as fast?
-    bash_parser: RwLock<Parser>,
-    pwsh_parser: RwLock<Parser>,
+    bash: Language,
+    pwsh: Language,
 
     // cached queries
-    bash_redirect_query: SpannedQuery,
-    bash_pipeline_query: SpannedQuery,
-    pwsh_redirect_query: SpannedQuery,
-    pwsh_pipeline_query: SpannedQuery,
+    bash_redirect_query: utils::SpannedQuery,
+    bash_pipeline_query: utils::SpannedQuery,
+    pwsh_redirect_query: utils::SpannedQuery,
+    pwsh_pipeline_query: utils::SpannedQuery,
 }
 
 audit_meta!(GitHubEnv, "github-env", "dangerous use of environment file");
 
-/// Holds a tree-sitter query that contains a `@span` capture that
-/// covers the entire range of the query.
-struct SpannedQuery {
-    inner: Query,
-    span_idx: u32,
-    destination_idx: u32,
-}
-
-impl Deref for SpannedQuery {
-    type Target = Query;
-
-    fn deref(&self) -> &Self::Target {
-        &self.inner
-    }
-}
-
-impl SpannedQuery {
-    fn new(query: &'static str, language: &Language) -> Self {
-        let query = Query::new(language, query).expect("malformed query");
-        let span_idx = query.capture_index_for_name("span").unwrap();
-        let destination_idx = query.capture_index_for_name("destination").unwrap();
-
-        Self {
-            inner: query,
-            span_idx,
-            destination_idx,
-        }
-    }
-}
-
 const BASH_REDIRECT_QUERY: &str = r#"
 (redirected_statement
  (
@@ -168,7 +134,7 @@ impl GitHubEnv {
 
     fn query<'a>(
         &self,
-        query: &'a SpannedQuery,
+        query: &'a utils::SpannedQuery,
         cursor: &'a mut QueryCursor,
         tree: &'a Tree,
         source: &'a str,
@@ -180,12 +146,14 @@ impl GitHubEnv {
         &self,
         script_body: &'hay str,
     ) -> Result<Vec<(&'hay str, Range<usize>)>> {
+        let mut parser = Parser::new();
+        parser
+            .set_language(&self.bash)
+            .context("failed to set bash language for parser")?;
+
         let mut cursor = QueryCursor::new();
 
-        let tree = self
-            .bash_parser
-            .write()
-            .unwrap()
+        let tree = parser
             .parse(script_body, None)
             .context("failed to parse `run:` body as bash")?;
 
@@ -283,10 +251,12 @@ impl GitHubEnv {
         &self,
         script_body: &'hay str,
     ) -> Result<Vec<(&'hay str, Range<usize>)>> {
-        let tree = &self
-            .pwsh_parser
-            .write()
-            .unwrap()
+        let mut parser = Parser::new();
+        parser
+            .set_language(&self.pwsh)
+            .context("failed to set pwsh language for parser")?;
+
+        let tree = parser
             .parse(script_body, None)
             .context("failed to parse `run:` body as pwsh")?;
 
@@ -295,7 +265,7 @@ impl GitHubEnv {
         let mut matching_spans = vec![];
 
         for query in queries {
-            let matches = self.query(query, &mut cursor, tree, script_body);
+            let matches = self.query(query, &mut cursor, &tree, script_body);
             matches.for_each(|mat| {
                 let span = mat
                     .captures
@@ -330,7 +300,8 @@ impl GitHubEnv {
         let normalized = utils::normalize_shell(shell);
 
         match normalized {
-            "bash" | "sh" => self.bash_uses_github_env(run_step_body),
+            // NOTE(ww): zsh is probably close enough in syntax to slide here. Hopefully.
+            "bash" | "sh" | "zsh" => self.bash_uses_github_env(run_step_body),
             "cmd" => Ok(self.cmd_uses_github_env(run_step_body)),
             "pwsh" | "powershell" => self.pwsh_uses_github_env(run_step_body),
             // TODO: handle python.
@@ -364,12 +335,12 @@ impl Audit for GitHubEnv {
             .map_err(AuditLoadError::Skip)?;
 
         Ok(Self {
-            bash_parser: RwLock::new(bash_parser),
-            pwsh_parser: RwLock::new(pwsh_parser),
-            bash_redirect_query: SpannedQuery::new(BASH_REDIRECT_QUERY, &bash),
-            bash_pipeline_query: SpannedQuery::new(BASH_PIPELINE_QUERY, &bash),
-            pwsh_redirect_query: SpannedQuery::new(PWSH_REDIRECT_QUERY, &pwsh),
-            pwsh_pipeline_query: SpannedQuery::new(PWSH_PIPELINE_QUERY, &pwsh),
+            bash_redirect_query: utils::SpannedQuery::new(BASH_REDIRECT_QUERY, &bash),
+            bash_pipeline_query: utils::SpannedQuery::new(BASH_PIPELINE_QUERY, &bash),
+            pwsh_redirect_query: utils::SpannedQuery::new(PWSH_REDIRECT_QUERY, &pwsh),
+            pwsh_pipeline_query: utils::SpannedQuery::new(PWSH_PIPELINE_QUERY, &pwsh),
+            bash,
+            pwsh,
         })
     }
 
@@ -410,7 +381,7 @@ impl Audit for GitHubEnv {
                         .add_location(
                             step.location()
                                 .primary()
-                                .with_keys(&["run".into()])
+                                .with_keys(["run".into()])
                                 .annotated(format!("write to {dest} may allow code execution")),
                         )
                         .build(step.workflow())?,
@@ -440,7 +411,7 @@ impl Audit for GitHubEnv {
                     .add_location(
                         step.location()
                             .primary()
-                            .with_keys(&["run".into()])
+                            .with_keys(["run".into()])
                             .annotated(format!("write to {dest} may allow code execution")),
                     )
                     .build(step.action())?,
@@ -523,8 +494,7 @@ mod tests {
             let audit_state = AuditState {
                 config: &Default::default(),
                 no_online_audits: false,
-                cache_dir: "/tmp/zizmor".into(),
-                gh_token: None,
+                gh_client: None,
                 gh_hostname: GitHubHost::Standard("github.com".into()),
             };
 
@@ -640,8 +610,7 @@ mod tests {
             let audit_state = AuditState {
                 config: &Default::default(),
                 no_online_audits: false,
-                cache_dir: "/tmp/zizmor".into(),
-                gh_token: None,
+                gh_client: None,
                 gh_hostname: GitHubHost::Standard("github.com".into()),
             };
 
