@@ -15,7 +15,7 @@
 //! A small amount of additional processing is done to remove template
 //! expressions that an attacker can't control.
 
-use std::{collections::HashMap, env, ops::Deref, sync::LazyLock, vec};
+use std::{env, ops::Deref, sync::LazyLock, vec};
 
 use fst::Map;
 use github_actions_expressions::{Expr, Literal, context::Context};
@@ -29,16 +29,17 @@ use super::{Audit, AuditLoadError, audit_meta};
 use crate::{
     finding::{
         Confidence, Finding, Fix, Persona, Severity,
-        location::{Routable as _, Subfeature, SymbolicLocation},
+        location::{Routable as _, SymbolicLocation},
     },
     models::{
         self, StepCommon, action::CompositeStep, inputs::Capability, uses::RepositoryUsesPattern,
         workflow::Step,
     },
     state::AuditState,
-    utils::{DEFAULT_ENVIRONMENT_VARIABLES, ExtractedExpr, extract_expressions},
-    yaml_patch::{Op, Patch},
+    utils::{self, DEFAULT_ENVIRONMENT_VARIABLES, ExtractedExpr, extract_fenced_expressions},
 };
+use subfeature::Subfeature;
+use yamlpatch::{Op, Patch};
 
 pub(crate) struct TemplateInjection;
 
@@ -55,16 +56,22 @@ static ACTION_INJECTION_SINKS: LazyLock<Vec<(RepositoryUsesPattern, Vec<&str>)>>
         )
         .unwrap();
 
-        // These sinks are not tracked by CodeQL (yet)
-        sinks.push(("amadevus/pwsh-script".parse().unwrap(), vec!["script"]));
-        sinks.push((
-            "jannekem/run-python-script-action".parse().unwrap(),
-            vec!["script"],
-        ));
-        sinks.push((
-            "cardinalby/js-eval-action".parse().unwrap(),
-            vec!["expression"],
-        ));
+        sinks.extend([
+            // These sinks are not tracked by CodeQL (yet)
+            ("amadevus/pwsh-script".parse().unwrap(), vec!["script"]),
+            (
+                "jannekem/run-python-script-action".parse().unwrap(),
+                vec!["script"],
+            ),
+            (
+                "cardinalby/js-eval-action".parse().unwrap(),
+                vec!["expression"],
+            ),
+            (
+                "addnab/docker-run-action".parse().unwrap(),
+                vec!["options", "run"],
+            ),
+        ]);
         sinks
     });
 
@@ -130,16 +137,16 @@ impl TemplateInjection {
                         .map(|script| {
                             (
                                 script,
-                                step.location().with_keys(&["with".into(), input.into()]),
+                                step.location().with_keys(["with".into(), input.into()]),
                                 vec![
                                     // TODO: Plumb the step name/id as a related
                                     // location here and below; this will require us
                                     // to add it to StepCommon.
                                     step.location()
-                                        .with_keys(&["uses".into()])
+                                        .with_keys(["uses".into()])
                                         .annotated("action accepts arbitrary code"),
                                     step.location()
-                                        .with_keys(&["with".into(), input.into()])
+                                        .with_keys(["with".into(), input.into()])
                                         .annotated("via this input")
                                         .key_only(),
                                 ],
@@ -150,10 +157,10 @@ impl TemplateInjection {
             models::StepBodyCommon::Run { run, .. } => {
                 vec![(
                     run,
-                    step.location().with_keys(&["run".into()]),
+                    step.location().with_keys(["run".into()]),
                     vec![
                         step.location()
-                            .with_keys(&["run".into()])
+                            .with_keys(["run".into()])
                             .annotated("this run block")
                             .key_only(),
                     ],
@@ -163,6 +170,28 @@ impl TemplateInjection {
         }
     }
 
+    /// Returns the appropriate variable expansion syntax for the given variable
+    /// based on the step's shell type. Returns `None` if the shell type is unsupported
+    /// for auto-fixing.
+    fn variable_expansion_for_shell<'doc>(
+        env_var: &str,
+        step: &impl StepCommon<'doc>,
+    ) -> Option<String> {
+        // Only provide fixes for run steps
+        if !matches!(step.body(), models::StepBodyCommon::Run { .. }) {
+            return None;
+        }
+
+        let shell = utils::normalize_shell(step.shell()?);
+
+        match shell {
+            "bash" | "sh" | "zsh" => Some(format!("${{{env_var}}}")),
+            "cmd" => Some(format!("%{env_var}%")),
+            "pwsh" | "powershell" => Some(format!("$env:{env_var}")),
+            _ => None,
+        }
+    }
+
     /// Converts a [`Context`] into an appropriate environment variable name,
     /// or `None` if conversion is not possible.
     fn context_to_env_var(ctx: &Context) -> Option<String> {
@@ -257,9 +286,6 @@ impl TemplateInjection {
             return None;
         }
 
-        // FIXME: We should only produce a fix if we're confident that
-        // the `run:` block has bash syntax.
-
         // If our expression isn't a single context, then we can't fix it yet.
         let Expr::Context(ctx) = parsed else {
             return None;
@@ -267,8 +293,9 @@ impl TemplateInjection {
 
         // From here, our fix consists of two patch operations:
         // 1. Replacing the expression in the script with an environment
-        //    variable of our generation. For example, `${{ foo.bar }}`
-        //    becomes `${FOO_BAR}`.
+        //    variable of our generation. The variable syntax depends on
+        //    the shell type: `${VAR}` for bash/sh, `%VAR%` for cmd,
+        //    `$env:VAR` for PowerShell.
         // 2. Inserting the new environment variable into the step's
         //    `env:` block, e.g. `FOO_BAR: ${{ foo.bar }}`.
         //
@@ -282,29 +309,37 @@ impl TemplateInjection {
         // index. In those kinds of cases, we don't produce a fix.
         let env_var = Self::context_to_env_var(ctx)?;
 
+        // Express the variable's expansion according to the step's shell.
+        // For example, `VAR` becomes `${VAR}` in bash/sh/zsh,
+        let var_expansion = Self::variable_expansion_for_shell(&env_var, step)?;
+
         let mut patches = vec![];
         patches.push(Patch {
-            route: step.route().with_keys(&["run".into()]),
+            route: step.route().with_key("run"),
             operation: Op::RewriteFragment {
-                from: raw.as_raw().to_string().into(),
-                to: format!("${{{env_var}}}").into(),
-                after: None,
+                from: subfeature::Subfeature::new(0, raw.as_raw()),
+                to: var_expansion.into(),
             },
         });
 
-        // We only need to add the environment variable if it doesn't
-        // match one of the runner's default environment variables.
-        if !DEFAULT_ENVIRONMENT_VARIABLES
+        // We need to insert a new key into the `env:` block, unless the
+        // variable is already a default *or* the context is `env.FOO`,
+        // since the latter implies that `FOO` is already present.
+        let needs_new_env = !DEFAULT_ENVIRONMENT_VARIABLES
             .iter()
             .map(|t| t.0)
             .contains(&env_var.as_str())
-        {
+            && !ctx.child_of("env");
+
+        if needs_new_env {
             patches.push(Patch {
                 route: step.route(),
                 operation: Op::MergeInto {
                     key: "env".to_string(),
-                    value: serde_yaml::to_value(HashMap::from([(env_var.as_str(), raw.as_raw())]))
-                        .unwrap(),
+                    updates: indexmap::IndexMap::from_iter([(
+                        env_var.clone(),
+                        serde_yaml::Value::String(raw.as_raw().into()),
+                    )]),
                 },
             });
         }
@@ -329,7 +364,7 @@ impl TemplateInjection {
         Persona,
     )> {
         let mut bad_expressions = vec![];
-        for (expr, expr_span) in extract_expressions(script) {
+        for (expr, expr_span) in extract_fenced_expressions(script) {
             let Ok(parsed) = Expr::parse(expr.as_bare()) else {
                 tracing::warn!("couldn't parse expression: {expr}", expr = expr.as_raw());
                 continue;
@@ -603,8 +638,7 @@ mod tests {
             let audit_state = AuditState {
                 config: &Default::default(),
                 no_online_audits: false,
-                cache_dir: "/tmp/zizmor".into(),
-                gh_token: None,
+                gh_client: None,
                 gh_hostname: GitHubHost::Standard("github.com".into()),
             };
             let audit = <$audit_type>::new(&audit_state).unwrap();
@@ -626,9 +660,7 @@ mod tests {
             .fixes
             .iter()
             .find(|f| f.title == expected_title)
-            .unwrap_or_else(|| {
-                panic!("Expected fix with title '{}' but not found", expected_title)
-            });
+            .unwrap_or_else(|| panic!("Expected fix with title '{expected_title}' but not found"));
 
         fix.apply(document).unwrap()
     }
@@ -781,7 +813,7 @@ jobs:
                           - name: Vulnerable step with existing env
                             run: echo "Event name is ${GITHUB_EVENT_HEAD_COMMIT_MESSAGE}"
                             env:
-                              EXISTING_VAR: existing_value
+                              EXISTING_VAR: "existing_value"
                               GITHUB_EVENT_HEAD_COMMIT_MESSAGE: ${{ github.event.head_commit.message }}
                     "#);
                 }
@@ -1024,6 +1056,66 @@ jobs:
         );
     }
 
+    #[test]
+    fn test_template_injection_fix_no_env_overcorrection() {
+        // Testcase for #1052.
+        let workflow_content = r#"
+name: Test Duplicate Template Injections
+on: push
+jobs:
+  test:
+    runs-on: ubuntu-latest
+    steps:
+      - name: Equivalent vulnerable expressions
+        run: |
+          echo "User: ${{ env.THIS_IS_NOT_A_DEFAULT }}"
+"#;
+
+        test_workflow_audit!(
+            TemplateInjection,
+            "test_template_injection_fix_no_env_overcorrection.yml",
+            workflow_content,
+            |workflow: &Workflow, findings: Vec<crate::finding::Finding>| {
+                // Should find template injection
+                assert!(!findings.is_empty());
+
+                // Should have at least one finding with a fix
+                let findings_with_fixes: Vec<_> =
+                    findings.iter().filter(|f| !f.fixes.is_empty()).collect();
+                assert!(
+                    !findings_with_fixes.is_empty(),
+                    "Expected at least one finding with a fix"
+                );
+
+                // Apply each fix in sequence
+                let mut current_document = workflow.as_document().clone();
+                for finding in findings_with_fixes {
+                    if let Some(fix) = finding
+                        .fixes
+                        .iter()
+                        .find(|f| f.title == "replace expression with environment variable")
+                    {
+                        if let Ok(new_document) = fix.apply(&current_document) {
+                            current_document = new_document;
+                        }
+                    }
+                }
+
+                insta::assert_snapshot!(current_document.source(), @r#"
+                name: Test Duplicate Template Injections
+                on: push
+                jobs:
+                  test:
+                    runs-on: ubuntu-latest
+                    steps:
+                      - name: Equivalent vulnerable expressions
+                        run: |
+                          echo "User: ${THIS_IS_NOT_A_DEFAULT}"
+                "#);
+            }
+        );
+    }
+
     #[test]
     fn test_capability_from_context() {
         assert!(matches!(
@@ -1094,4 +1186,358 @@ jobs:
             );
         }
     }
+
+    #[test]
+    fn test_template_injection_fix_bash_shell() {
+        let workflow_content = r#"
+name: Test Template Injection - Bash
+on: push
+jobs:
+  test:
+    runs-on: ubuntu-latest
+    steps:
+      - name: Vulnerable step with bash shell
+        shell: bash
+        run: echo "User is ${{ github.actor }}"
+"#;
+
+        test_workflow_audit!(
+            TemplateInjection,
+            "test_template_injection_fix_bash_shell.yml",
+            workflow_content,
+            |workflow: &Workflow, findings: Vec<crate::finding::Finding>| {
+                assert!(!findings.is_empty());
+
+                let finding_with_fix = findings.iter().find(|f| !f.fixes.is_empty());
+                assert!(finding_with_fix.is_some());
+
+                if let Some(finding) = finding_with_fix {
+                    let fixed_content = apply_fix_by_title_for_snapshot(
+                        workflow.as_document(),
+                        finding,
+                        "replace expression with environment variable",
+                    );
+                    insta::assert_snapshot!(fixed_content.source(), @r#"
+                    name: Test Template Injection - Bash
+                    on: push
+                    jobs:
+                      test:
+                        runs-on: ubuntu-latest
+                        steps:
+                          - name: Vulnerable step with bash shell
+                            shell: bash
+                            run: echo "User is ${GITHUB_ACTOR}"
+                    "#);
+                }
+            }
+        );
+    }
+
+    #[test]
+    fn test_template_injection_fix_bash_shell_full_path() {
+        let workflow_content = r#"
+name: Test Template Injection - Bash
+on: push
+jobs:
+  test:
+    runs-on: ubuntu-latest
+    steps:
+      - name: Vulnerable step with bash shell
+        shell: /bin/bash
+        run: echo "User is ${{ github.actor }}"
+"#;
+
+        test_workflow_audit!(
+            TemplateInjection,
+            "test_template_injection_fix_bash_shell.yml",
+            workflow_content,
+            |workflow: &Workflow, findings: Vec<crate::finding::Finding>| {
+                assert!(!findings.is_empty());
+
+                let finding_with_fix = findings.iter().find(|f| !f.fixes.is_empty());
+                assert!(finding_with_fix.is_some());
+
+                if let Some(finding) = finding_with_fix {
+                    let fixed_content = apply_fix_by_title_for_snapshot(
+                        workflow.as_document(),
+                        finding,
+                        "replace expression with environment variable",
+                    );
+                    insta::assert_snapshot!(fixed_content.source(), @r#"
+                    name: Test Template Injection - Bash
+                    on: push
+                    jobs:
+                      test:
+                        runs-on: ubuntu-latest
+                        steps:
+                          - name: Vulnerable step with bash shell
+                            shell: /bin/bash
+                            run: echo "User is ${GITHUB_ACTOR}"
+                    "#);
+                }
+            }
+        );
+    }
+
+    #[test]
+    fn test_template_injection_fix_cmd_shell() {
+        let workflow_content = r#"
+name: Test Template Injection - CMD
+on: push
+jobs:
+  test:
+    runs-on: windows-latest
+    steps:
+      - name: Vulnerable step with cmd shell
+        shell: cmd
+        run: echo User is ${{ github.actor }}
+"#;
+
+        test_workflow_audit!(
+            TemplateInjection,
+            "test_template_injection_fix_cmd_shell.yml",
+            workflow_content,
+            |workflow: &Workflow, findings: Vec<crate::finding::Finding>| {
+                assert!(!findings.is_empty());
+
+                let finding_with_fix = findings.iter().find(|f| !f.fixes.is_empty());
+                assert!(finding_with_fix.is_some());
+
+                if let Some(finding) = finding_with_fix {
+                    let fixed_content = apply_fix_by_title_for_snapshot(
+                        workflow.as_document(),
+                        finding,
+                        "replace expression with environment variable",
+                    );
+                    insta::assert_snapshot!(fixed_content.source(), @r#"
+                    name: Test Template Injection - CMD
+                    on: push
+                    jobs:
+                      test:
+                        runs-on: windows-latest
+                        steps:
+                          - name: Vulnerable step with cmd shell
+                            shell: cmd
+                            run: echo User is %GITHUB_ACTOR%
+                    "#);
+                }
+            }
+        );
+    }
+
+    #[test]
+    fn test_template_injection_fix_pwsh_shell() {
+        let workflow_content = r#"
+name: Test Template Injection - PowerShell
+on: push
+jobs:
+  test:
+    runs-on: windows-latest
+    steps:
+      - name: Vulnerable step with pwsh shell
+        shell: pwsh
+        run: Write-Host "User is ${{ github.actor }}"
+"#;
+
+        test_workflow_audit!(
+            TemplateInjection,
+            "test_template_injection_fix_pwsh_shell.yml",
+            workflow_content,
+            |workflow: &Workflow, findings: Vec<crate::finding::Finding>| {
+                assert!(!findings.is_empty());
+
+                let finding_with_fix = findings.iter().find(|f| !f.fixes.is_empty());
+                assert!(finding_with_fix.is_some());
+
+                if let Some(finding) = finding_with_fix {
+                    let fixed_content = apply_fix_by_title_for_snapshot(
+                        workflow.as_document(),
+                        finding,
+                        "replace expression with environment variable",
+                    );
+                    insta::assert_snapshot!(fixed_content.source(), @r#"
+                    name: Test Template Injection - PowerShell
+                    on: push
+                    jobs:
+                      test:
+                        runs-on: windows-latest
+                        steps:
+                          - name: Vulnerable step with pwsh shell
+                            shell: pwsh
+                            run: Write-Host "User is $env:GITHUB_ACTOR"
+                    "#);
+                }
+            }
+        );
+    }
+
+    #[test]
+    fn test_template_injection_fix_default_shell_ubuntu() {
+        let workflow_content = r#"
+name: Test Template Injection - Default Shell Ubuntu
+on: push
+jobs:
+  test:
+    runs-on: ubuntu-latest
+    steps:
+      - name: Vulnerable step with default shell
+        run: echo "User is ${{ github.actor }}"
+"#;
+
+        test_workflow_audit!(
+            TemplateInjection,
+            "test_template_injection_fix_default_shell_ubuntu.yml",
+            workflow_content,
+            |workflow: &Workflow, findings: Vec<crate::finding::Finding>| {
+                assert!(!findings.is_empty());
+
+                let finding_with_fix = findings.iter().find(|f| !f.fixes.is_empty());
+                assert!(finding_with_fix.is_some());
+
+                if let Some(finding) = finding_with_fix {
+                    let fixed_content = apply_fix_by_title_for_snapshot(
+                        workflow.as_document(),
+                        finding,
+                        "replace expression with environment variable",
+                    );
+                    // Ubuntu default shell is bash, so should use ${VAR} syntax
+                    insta::assert_snapshot!(fixed_content.source(), @r#"
+                    name: Test Template Injection - Default Shell Ubuntu
+                    on: push
+                    jobs:
+                      test:
+                        runs-on: ubuntu-latest
+                        steps:
+                          - name: Vulnerable step with default shell
+                            run: echo "User is ${GITHUB_ACTOR}"
+                    "#);
+                }
+            }
+        );
+    }
+
+    #[test]
+    fn test_template_injection_fix_default_shell_windows() {
+        let workflow_content = r#"
+name: Test Template Injection - Default Shell Windows
+on: push
+jobs:
+  test:
+    runs-on: windows-latest
+    steps:
+      - name: Vulnerable step with default shell
+        run: Write-Host "User is ${{ github.actor }}"
+"#;
+
+        test_workflow_audit!(
+            TemplateInjection,
+            "test_template_injection_fix_default_shell_windows.yml",
+            workflow_content,
+            |workflow: &Workflow, findings: Vec<crate::finding::Finding>| {
+                assert!(!findings.is_empty());
+
+                let finding_with_fix = findings.iter().find(|f| !f.fixes.is_empty());
+                assert!(finding_with_fix.is_some());
+
+                if let Some(finding) = finding_with_fix {
+                    let fixed_content = apply_fix_by_title_for_snapshot(
+                        workflow.as_document(),
+                        finding,
+                        "replace expression with environment variable",
+                    );
+                    // Windows default shell is pwsh, so should use $env:VAR syntax
+                    insta::assert_snapshot!(fixed_content.source(), @r#"
+                    name: Test Template Injection - Default Shell Windows
+                    on: push
+                    jobs:
+                      test:
+                        runs-on: windows-latest
+                        steps:
+                          - name: Vulnerable step with default shell
+                            run: Write-Host "User is $env:GITHUB_ACTOR"
+                    "#);
+                }
+            }
+        );
+    }
+
+    #[test]
+    fn test_template_injection_fix_cmd_shell_with_custom_env() {
+        let workflow_content = r#"
+name: Test Template Injection - CMD with Custom Env
+on: push
+jobs:
+  test:
+    runs-on: windows-latest
+    steps:
+      - name: Vulnerable step with custom context
+        shell: cmd
+        run: echo PR title is ${{ github.event.pull_request.title }}
+"#;
+
+        test_workflow_audit!(
+            TemplateInjection,
+            "test_template_injection_fix_cmd_shell_with_custom_env.yml",
+            workflow_content,
+            |workflow: &Workflow, findings: Vec<crate::finding::Finding>| {
+                assert!(!findings.is_empty());
+
+                let finding_with_fix = findings.iter().find(|f| !f.fixes.is_empty());
+                assert!(finding_with_fix.is_some());
+
+                if let Some(finding) = finding_with_fix {
+                    let fixed_content = apply_fix_by_title_for_snapshot(
+                        workflow.as_document(),
+                        finding,
+                        "replace expression with environment variable",
+                    );
+                    insta::assert_snapshot!(fixed_content.source(), @r#"
+                    name: Test Template Injection - CMD with Custom Env
+                    on: push
+                    jobs:
+                      test:
+                        runs-on: windows-latest
+                        steps:
+                          - name: Vulnerable step with custom context
+                            shell: cmd
+                            run: echo PR title is %GITHUB_EVENT_PULL_REQUEST_TITLE%
+                            env:
+                              GITHUB_EVENT_PULL_REQUEST_TITLE: ${{ github.event.pull_request.title }}
+                    "#);
+                }
+            }
+        );
+    }
+
+    #[test]
+    fn test_template_injection_no_fix_unknown_shell() {
+        let workflow_content = r#"
+name: Test Template Injection - Unknown Shell
+on: push
+jobs:
+  test:
+    runs-on: ubuntu-latest
+    steps:
+      - name: Vulnerable step with unknown shell
+        shell: fish
+        run: echo "User is ${{ github.actor }}"
+"#;
+
+        test_workflow_audit!(
+            TemplateInjection,
+            "test_template_injection_no_fix_unknown_shell.yml",
+            workflow_content,
+            |_workflow: &Workflow, findings: Vec<crate::finding::Finding>| {
+                // Should find the vulnerability but not provide a fix for unknown shell
+                assert!(!findings.is_empty());
+
+                let findings_with_fixes: Vec<_> =
+                    findings.iter().filter(|f| !f.fixes.is_empty()).collect();
+                assert!(
+                    findings_with_fixes.is_empty(),
+                    "Expected no fixes for unknown shell type"
+                );
+            }
+        );
+    }
 }
