@@ -106,7 +106,7 @@ impl UnpinnedUses {
                     .add_location(
                         step.location()
                             .primary()
-                            .with_keys(&["uses".into()])
+                            .with_keys(["uses".into()])
                             .annotated(annotation),
                     )
                     .build(step)?,
