@@ -5,11 +5,8 @@ use clap::ValueEnum;
 use serde::{Deserialize, Serialize};
 
 use self::location::{Location, SymbolicLocation};
-use crate::{
-    InputKey,
-    models::AsDocument,
-    yaml_patch::{self, Patch},
-};
+use crate::{InputKey, models::AsDocument};
+use yamlpatch::{self, Patch};
 
 pub(crate) mod location;
 
@@ -97,7 +94,7 @@ pub(crate) enum Severity {
 }
 
 /// A finding's "determination," i.e. its various classifications.
-#[derive(Serialize)]
+#[derive(Copy, Clone, Serialize)]
 pub(crate) struct Determinations {
     pub(crate) confidence: Confidence,
     pub(crate) severity: Severity,
@@ -137,14 +134,13 @@ impl Fix<'_> {
         &self,
         document: &yamlpath::Document,
     ) -> anyhow::Result<yamlpath::Document> {
-        match yaml_patch::apply_yaml_patches(document, &self.patches) {
+        match yamlpatch::apply_yaml_patches(document, &self.patches) {
             Ok(new_document) => Ok(new_document),
             Err(e) => Err(anyhow!("fix failed: {e}")),
         }
     }
 }
 
-#[derive(Serialize)]
 pub(crate) struct Finding<'doc> {
     /// The audit ID for this finding, e.g. `template-injection`.
     pub(crate) ident: &'static str,
@@ -166,7 +162,6 @@ pub(crate) struct Finding<'doc> {
     /// One or more suggested fixes for this finding. Because a finding
     /// can span multiple inputs, each fix is associated with a specific
     /// input via [`Fix::key`].
-    #[serde(skip_serializing)]
     pub(crate) fixes: Vec<Fix<'doc>>,
 }
 
