@@ -9,14 +9,13 @@ use std::{
 use annotate_snippets::{Level, Renderer};
 use anstream::{eprintln, println, stream::IsTerminal};
 use anyhow::{Context, Result, anyhow};
-use camino::{Utf8Path, Utf8PathBuf};
-use clap::{Args, CommandFactory, Parser, ValueEnum};
+use camino::Utf8Path;
+use clap::{Args, CommandFactory, Parser, ValueEnum, builder::NonEmptyStringValueParser};
 use clap_complete::Generator;
 use clap_verbosity_flag::InfoLevel;
 use config::Config;
 use finding::{Confidence, Persona, Severity};
-use github_actions_models::common::Uses;
-use github_api::GitHubHost;
+use github_api::{GitHubHost, GitHubToken};
 use ignore::WalkBuilder;
 use indicatif::ProgressStyle;
 use owo_colors::OwoColorize;
@@ -27,6 +26,8 @@ use tracing::{Span, info_span, instrument};
 use tracing_indicatif::{IndicatifLayer, span_ext::IndicatifSpanExt};
 use tracing_subscriber::{EnvFilter, layer::SubscriberExt as _, util::SubscriberInitExt as _};
 
+use crate::registry::RepoSlug;
+
 mod audit;
 mod config;
 mod finding;
@@ -38,7 +39,6 @@ mod output;
 mod registry;
 mod state;
 mod utils;
-mod yaml_patch;
 
 // TODO: Dedupe this with the top-level `sponsors.json` used by the
 // README + docs site.
@@ -70,11 +70,11 @@ struct App {
     offline: bool,
 
     /// The GitHub API token to use.
-    #[arg(long, env)]
-    gh_token: Option<String>,
+    #[arg(long, env, value_parser = GitHubToken::new)]
+    gh_token: Option<GitHubToken>,
 
     /// The GitHub Server Hostname. Defaults to github.com
-    #[arg(long, env = "GH_HOST", default_value = "github.com", value_parser = GitHubHost::from_clap)]
+    #[arg(long, env = "GH_HOST", default_value = "github.com", value_parser = GitHubHost::new)]
     gh_hostname: GitHubHost,
 
     /// Perform only offline audits.
@@ -102,8 +102,14 @@ struct App {
 
     /// The configuration file to load. By default, any config will be
     /// discovered relative to $CWD.
-    #[arg(short, long, env = "ZIZMOR_CONFIG", group = "conf")]
-    config: Option<Utf8PathBuf>,
+    #[arg(
+        short,
+        long,
+        env = "ZIZMOR_CONFIG",
+        group = "conf",
+        value_parser = NonEmptyStringValueParser::new()
+    )]
+    config: Option<String>,
 
     /// Disable all configuration loading.
     #[arg(long, group = "conf")]
@@ -124,7 +130,7 @@ struct App {
     /// The directory to use for HTTP caching. By default, a
     /// host-appropriate user-caching directory will be used.
     #[arg(long)]
-    cache_dir: Option<Utf8PathBuf>,
+    cache_dir: Option<String>,
 
     /// Control which kinds of inputs are collected for auditing.
     ///
@@ -415,8 +421,7 @@ fn collect_from_repo_slug(
     state: &AuditState,
     registry: &mut InputRegistry,
 ) -> Result<()> {
-    // Our pre-existing `uses: <slug>` parser does 90% of the work for us.
-    let Ok(Uses::Repository(slug)) = Uses::from_str(input) else {
+    let Ok(slug) = RepoSlug::from_str(input) else {
         return Err(anyhow!(tips(
             format!("invalid input: {input}"),
             &[format!(
@@ -428,15 +433,7 @@ fn collect_from_repo_slug(
         )));
     };
 
-    // We don't expect subpaths here.
-    if slug.subpath.is_some() {
-        return Err(anyhow!(tips(
-            "invalid GitHub repository reference",
-            &["pass owner/repo or owner/repo@ref"]
-        )));
-    }
-
-    let client = state.github_client().ok_or_else(|| {
+    let client = state.gh_client.as_ref().ok_or_else(|| {
         anyhow!(tips(
             format!("can't retrieve repository: {input}", input = input.green()),
             &[format!(
@@ -641,7 +638,7 @@ fn run() -> Result<ExitCode> {
         ))
     })?;
 
-    let audit_state = AuditState::new(&app, &config);
+    let audit_state = AuditState::new(&app, &config)?;
     let registry = collect_inputs(
         &app.inputs,
         &app.collect,
@@ -687,7 +684,7 @@ fn run() -> Result<ExitCode> {
     match app.format {
         OutputFormat::Plain => output::plain::render_findings(&app, &registry, &results),
         OutputFormat::Json | OutputFormat::JsonV1 => {
-            serde_json::to_writer_pretty(stdout(), &results.findings())?
+            output::json::v1::output(stdout(), results.findings())?
         }
         OutputFormat::Sarif => {
             serde_json::to_writer_pretty(stdout(), &output::sarif::build(results.findings()))?
