@@ -13,11 +13,13 @@
 // able to express things like
 // "match foo/bar if foo: A and not bar: B and baz: /abcd/"
 
-use std::ops::{BitAnd, BitOr};
+use std::ops::{BitAnd, BitOr, Not};
 
-use github_actions_models::common::{EnvValue, Uses, expr::ExplicitExpr};
+use github_actions_models::common::{EnvValue, Uses};
 use indexmap::IndexMap;
 
+use crate::utils::ExtractedExpr;
+
 use super::{StepBodyCommon, StepCommon, uses::RepositoryUsesPattern};
 
 pub(crate) enum ActionCoordinate {
@@ -44,7 +46,7 @@ impl ActionCoordinate {
     /// while the `Some(_)` variants indicate various (potential) usages (such as being implicitly
     /// enabled, or explicitly enabled, or potentially enabled by a template expansion that
     /// can't be directly analyzed).
-    pub(crate) fn usage<'s>(&self, step: &impl StepCommon<'s>) -> Option<Usage> {
+    pub(crate) fn usage<'doc>(&self, step: &impl StepCommon<'doc>) -> Option<Usage> {
         let uses_pattern = self.uses_pattern();
 
         let StepBodyCommon::Uses {
@@ -89,8 +91,14 @@ pub(crate) enum Toggle {
 pub(crate) enum ControlFieldType {
     /// The behavior is controlled by a boolean field, e.g. `cache: true`.
     Boolean,
-    /// The behavior is controlled by a string field, e.g. `cache: "pip"`.
-    String,
+    /// The behavior is controlled by a "free" string field.
+    ///
+    /// This is effectively a "presence" check, i.e. is satisfied if
+    /// the field is present, regardless of its value.
+    FreeString,
+    /// The behavior is controlled by a "fixed" string field, i.e. only applies
+    /// when the field matches one of the given values.
+    Exact(&'static [&'static str]),
 }
 
 /// The result of evaluating a control expression.
@@ -107,6 +115,19 @@ pub(crate) enum ControlEvaluation {
     Conditional,
 }
 
+impl Not for ControlEvaluation {
+    type Output = Self;
+
+    fn not(self) -> Self::Output {
+        match self {
+            ControlEvaluation::DefaultSatisfied => ControlEvaluation::NotSatisfied,
+            ControlEvaluation::Satisfied => ControlEvaluation::NotSatisfied,
+            ControlEvaluation::NotSatisfied => ControlEvaluation::Satisfied,
+            ControlEvaluation::Conditional => ControlEvaluation::Conditional,
+        }
+    }
+}
+
 impl BitAnd for ControlEvaluation {
     type Output = Self;
 
@@ -248,6 +269,8 @@ pub(crate) enum ControlExpr {
     /// Existential quantification: any of the fields must be satisfied.
     #[allow(dead_code)]
     Any(Vec<ControlExpr>),
+    /// Negation: the "opposite" of the expression's satisfaction.
+    Not(Box<ControlExpr>),
 }
 
 impl ControlExpr {
@@ -269,6 +292,10 @@ impl ControlExpr {
         Self::All(exprs.into_iter().collect())
     }
 
+    pub(crate) fn not(expr: ControlExpr) -> Self {
+        Self::Not(Box::new(expr))
+    }
+
     pub(crate) fn eval(&self, with: &IndexMap<String, EnvValue>) -> ControlEvaluation {
         match self {
             ControlExpr::Single {
@@ -279,20 +306,50 @@ impl ControlExpr {
             } => {
                 // If the controlling field is not present, the default dictates the semantics.
                 if let Some(field_value) = with.get(*field_name) {
-                    match field_value.to_string().as_str() {
-                        "false" if matches!(field_type, ControlFieldType::Boolean) => {
-                            match toggle {
-                                Toggle::OptIn => ControlEvaluation::NotSatisfied,
-                                Toggle::OptOut => ControlEvaluation::Satisfied,
-                            }
-                        }
-                        other => match ExplicitExpr::from_curly(other) {
-                            None => match toggle {
+                    match field_type {
+                        // We expect a boolean control.
+                        ControlFieldType::Boolean => match field_value.to_string().as_str() {
+                            "true" => match toggle {
                                 Toggle::OptIn => ControlEvaluation::Satisfied,
                                 Toggle::OptOut => ControlEvaluation::NotSatisfied,
                             },
-                            Some(_) => ControlEvaluation::Conditional,
+                            "false" => match toggle {
+                                Toggle::OptIn => ControlEvaluation::NotSatisfied,
+                                Toggle::OptOut => ControlEvaluation::Satisfied,
+                            },
+                            other => match ExtractedExpr::from_fenced(other) {
+                                // We have something like `foo: ${{ expr }}`,
+                                // which could evaluate either way.
+                                Some(_) => ControlEvaluation::Conditional,
+                                // We have something like `foo: bar`, but we
+                                // were expecting a boolean. Assume pessimistically
+                                // that the action coerces any non-`false` value to `true`.
+                                None => match toggle {
+                                    Toggle::OptIn => ControlEvaluation::Satisfied,
+                                    Toggle::OptOut => ControlEvaluation::NotSatisfied,
+                                },
+                            },
                         },
+                        // We expect a "free" string control, i.e. any value.
+                        // Evaluate just the toggle.
+                        ControlFieldType::FreeString => match toggle {
+                            Toggle::OptIn => ControlEvaluation::Satisfied,
+                            Toggle::OptOut => ControlEvaluation::NotSatisfied,
+                        },
+                        // We expect a "fixed" string control, i.e. one of a set of values.
+                        ControlFieldType::Exact(items) => {
+                            if items.contains(&field_value.to_string().as_str()) {
+                                match toggle {
+                                    Toggle::OptIn => ControlEvaluation::Satisfied,
+                                    Toggle::OptOut => ControlEvaluation::NotSatisfied,
+                                }
+                            } else {
+                                match toggle {
+                                    Toggle::OptIn => ControlEvaluation::NotSatisfied,
+                                    Toggle::OptOut => ControlEvaluation::Satisfied,
+                                }
+                            }
+                        }
                     }
                 } else if *enabled_by_default {
                     ControlEvaluation::DefaultSatisfied
@@ -308,6 +365,7 @@ impl ControlExpr {
                 .iter()
                 .map(|expr| expr.eval(with))
                 .fold(ControlEvaluation::NotSatisfied, |acc, expr| acc | expr),
+            ControlExpr::Not(expr) => !expr.eval(with),
         }
     }
 }
@@ -324,73 +382,72 @@ pub(crate) enum Usage {
 mod tests {
     use std::str::FromStr;
 
-    use github_actions_models::workflow::job::Step;
-
-    use super::{ActionCoordinate, StepCommon};
-    use crate::models::{
-        coordinate::{ControlExpr, ControlFieldType, Toggle, Usage},
-        uses::RepositoryUsesPattern,
+    use super::ActionCoordinate;
+    use crate::{
+        models::{
+            coordinate::{ControlExpr, ControlFieldType, Toggle, Usage},
+            uses::RepositoryUsesPattern,
+            workflow::{Job, Workflow},
+        },
+        registry::InputKey,
     };
 
-    // Test-only trait impl.
-    impl<'s> StepCommon<'s> for Step {
-        fn env_is_static(&self, _name: &str) -> bool {
-            unimplemented!()
-        }
-
-        fn uses(&self) -> Option<&github_actions_models::common::Uses> {
-            unimplemented!()
-        }
-
-        fn strategy(&self) -> Option<&github_actions_models::workflow::job::Strategy> {
-            unimplemented!()
-        }
-
-        fn body(&self) -> super::StepBodyCommon {
-            match &self.body {
-                github_actions_models::workflow::job::StepBody::Uses { uses, with } => {
-                    super::StepBodyCommon::Uses { uses, with }
-                }
-                github_actions_models::workflow::job::StepBody::Run {
-                    run,
-                    working_directory,
-                    shell,
-                    env,
-                } => super::StepBodyCommon::Run {
-                    run,
-                    _working_directory: working_directory.as_deref(),
-                    _shell: shell.as_deref(),
-                    _env: env,
-                },
-            }
-        }
-
-        fn location(&self) -> crate::models::SymbolicLocation<'s> {
-            unimplemented!()
-        }
-
-        fn location_with_name(&self) -> crate::finding::SymbolicLocation<'s> {
-            unimplemented!()
-        }
-
-        fn document(&self) -> &'s yamlpath::Document {
-            unimplemented!()
-        }
-    }
-
     #[test]
     fn test_usage() {
+        let workflow = r#"
+    name: test_usage
+    on: push
+    jobs:
+      test_usage:
+        runs-on: ubuntu-latest
+        steps:
+          - uses: foo/bar@v1      # 0
+
+          - uses: foo/bar@v1      # 1
+
+          - uses: not/thesame@v1  # 2
+            with:
+              set-me: true
+
+          - uses: not/thesame@v1  # 3
+
+          - uses: foo/bar@v1      # 4
+            with:
+              set-me: true
+
+          - uses: foo/bar@v1      # 5
+            with:
+              set-me: false
+
+          - uses: foo/bar@v1      # 6
+            with:
+              disable-cache: true
+
+          - uses: foo/bar@v1      # 7
+            with:
+              disable-cache: false
+    "#;
+
+        let workflow =
+            Workflow::from_string(workflow.into(), InputKey::local("dummy", None).unwrap())
+                .unwrap();
+
+        let Job::NormalJob(job) = workflow.jobs().next().unwrap() else {
+            panic!("Expected a normal job");
+        };
+
+        let steps = job.steps().collect::<Vec<_>>();
+
         // Trivial case: no usage is possible, since the coordinate's `uses:`
         // does not match the step.
         let coord =
             ActionCoordinate::NotConfigurable(RepositoryUsesPattern::from_str("foo/bar").unwrap());
-        let step: Step = serde_yaml::from_str("uses: not/thesame").unwrap();
-        assert_eq!(coord.usage(&step), None);
+        let step = &steps[3];
+        assert_eq!(coord.usage(step), None);
 
         // Trivial cases: coordinate is not configurable and matches the `uses:`.
-        for step in &["uses: foo/bar", "uses: foo/bar@v1"] {
-            let step: Step = serde_yaml::from_str(step).unwrap();
-            assert_eq!(coord.usage(&step), Some(Usage::Always));
+        for step in &[&steps[0], &steps[1]] {
+            assert_eq!(coord.usage(*step), Some(Usage::Always));
         }
 
         // Coordinate `uses:` matches but is not enabled by default and is
@@ -399,32 +456,32 @@ mod tests {
             uses_pattern: RepositoryUsesPattern::from_str("foo/bar").unwrap(),
             control: ControlExpr::single(Toggle::OptIn, "set-me", ControlFieldType::Boolean, false),
         };
-        let step: Step = serde_yaml::from_str("uses: foo/bar").unwrap();
-        assert_eq!(coord.usage(&step), None);
+        let step = &steps[0];
+        assert_eq!(coord.usage(step), None);
 
         // Coordinate `uses:` matches and is explicitly toggled on.
-        let step: Step = serde_yaml::from_str("uses: foo/bar\nwith:\n  set-me: true").unwrap();
-        assert_eq!(coord.usage(&step), Some(Usage::DirectOptIn));
+        let step = &steps[4];
+        assert_eq!(coord.usage(step), Some(Usage::DirectOptIn));
 
         // Coordinate `uses:` matches but is explicitly toggled off.
-        let step: Step = serde_yaml::from_str("uses: foo/bar\nwith:\n  set-me: false").unwrap();
-        assert_eq!(coord.usage(&step), None);
+        let step = &steps[5];
+        assert_eq!(coord.usage(step), None);
 
         // Coordinate `uses:` matches and is enabled by default.
         let coord = ActionCoordinate::Configurable {
             uses_pattern: RepositoryUsesPattern::from_str("foo/bar").unwrap(),
             control: ControlExpr::single(Toggle::OptIn, "set-me", ControlFieldType::Boolean, true),
         };
-        let step: Step = serde_yaml::from_str("uses: foo/bar").unwrap();
-        assert_eq!(coord.usage(&step), Some(Usage::DefaultActionBehaviour));
+        let step = &steps[0];
+        assert_eq!(coord.usage(step), Some(Usage::DefaultActionBehaviour));
 
         // Coordinate `uses:` matches and is explicitly toggled on.
-        let step: Step = serde_yaml::from_str("uses: foo/bar\nwith:\n  set-me: true").unwrap();
-        assert_eq!(coord.usage(&step), Some(Usage::DirectOptIn));
+        let step = &steps[4];
+        assert_eq!(coord.usage(step), Some(Usage::DirectOptIn));
 
         // Coordinate `uses:` matches but is explicitly toggled off, despite default enablement.
-        let step: Step = serde_yaml::from_str("uses: foo/bar\nwith:\n  set-me: false").unwrap();
-        assert_eq!(coord.usage(&step), None);
+        let step = &steps[5];
+        assert_eq!(coord.usage(step), None);
 
         // Coordinate `uses:` matches and has an opt-out toggle, which does not affect
         // the default.
@@ -437,17 +494,15 @@ mod tests {
                 false,
             ),
         };
-        let step: Step = serde_yaml::from_str("uses: foo/bar").unwrap();
-        assert_eq!(coord.usage(&step), None);
+        let step = &steps[0];
+        assert_eq!(coord.usage(step), None);
 
         // Coordinate `uses:` matches and the opt-out inverts the match, clearing it.
-        let step: Step =
-            serde_yaml::from_str("uses: foo/bar\nwith:\n  disable-cache: true").unwrap();
-        assert_eq!(coord.usage(&step), None);
+        let step = &steps[6];
+        assert_eq!(coord.usage(step), None);
 
         // Coordinate `uses:` matches and the opt-out inverts the match, clearing it.
-        let step: Step =
-            serde_yaml::from_str("uses: foo/bar\nwith:\n  disable-cache: false").unwrap();
-        assert_eq!(coord.usage(&step), Some(Usage::DirectOptIn));
+        let step = &steps[7];
+        assert_eq!(coord.usage(step), Some(Usage::DirectOptIn));
     }
 }
