@@ -277,6 +277,12 @@ fn use_trusted_publishing() -> Result<()> {
             .run()?
     );
 
+    insta::assert_snapshot!(
+        zizmor()
+            .input(input_under_test("use-trusted-publishing/cargo-publish.yml"))
+            .run()?
+    );
+
     Ok(())
 }
 
@@ -531,6 +537,12 @@ fn cache_poisoning() -> Result<()> {
             .run()?
     );
 
+    insta::assert_snapshot!(
+        zizmor()
+            .input(input_under_test("cache-poisoning/issue-1081-repro.yml"))
+            .run()?
+    );
+
     Ok(())
 }
 
@@ -799,3 +811,14 @@ fn unpinned_images() -> Result<()> {
 
     Ok(())
 }
+
+#[test]
+fn unsound_condition() -> Result<()> {
+    insta::assert_snapshot!(
+        zizmor()
+            .input(input_under_test("unsound-condition.yml"))
+            .run()?
+    );
+
+    Ok(())
+}
